@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: Image.c,v 1.121 2007/05/23 17:50:57 dk Exp $
+ * $Id: Image.c,v 1.129 2008/04/09 07:12:33 dk Exp $
  */
 
 #include "img.h"
@@ -37,6 +37,14 @@
 #include <Image.inc>
 #include "Clipboard.h"
 
+#ifdef PerlIO
+typedef PerlIO *FileStream;
+#else
+#define PERLIO_IS_STDIO 1
+typedef FILE *FileStream;
+#define PerlIO_fileno(f) fileno(f)
+#endif
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -49,11 +57,17 @@ extern "C" {
 #define var (( PImage) self)
 
 static Bool Image_set_extended_data( Handle self, HV * profile);
+static void Image_reset_notifications( Handle self);
 
 void
 Image_init( Handle self, HV * profile)
 {
+   dPROFILE;
    inherited init( self, profile);
+   var-> eventMask1 =
+     ( query_method( self, "on_headerready", 0) ? IMG_EVENTS_HEADER_READY : 0) |
+     ( query_method( self, "on_dataready",   0) ? IMG_EVENTS_DATA_READY   : 0);
+   Image_reset_notifications( self);
    var->w = pget_i( width);
    var->h = pget_i( height);
    var->conversion = pget_i( conversion);
@@ -114,6 +128,26 @@ Image_init( Handle self, HV * profile)
    CORE_INIT_TRANSIENT(Image);
 }
 
+void
+Image_handle_event( Handle self, PEvent event)
+{
+   inherited handle_event ( self, event);
+   if ( var-> stage > csNormal) return;
+   switch ( event-> cmd) {
+   case cmImageHeaderReady:
+      my-> notify( self, "<s", "HeaderReady");
+      break;
+   case cmImageDataReady:
+      my-> update_change( self);
+      my-> notify( self, "<siiii", "DataReady", 
+         event-> gen. R. left,
+         event-> gen. R. bottom,
+         event-> gen. R. right - event-> gen. R. left   + 1,
+         event-> gen. R. top   - event-> gen. R. bottom + 1);
+      break;
+   }
+}
+
 void
 Image_reset( Handle self, int new_type, RGBColor * palette, int palSize)
 {
@@ -191,9 +225,9 @@ Image_stretch( Handle self, int width, int height)
    }
    lineSize = (( abs( width) * ( var->type & imBPP) + 31) / 32) * 4;
    newData = allocb( lineSize * abs( height));
-   memset( newData, 0, lineSize * abs( height));
    if ( newData == nil) 
          croak("Image::stretch: cannot allocate %d bytes", lineSize * abs( height));
+   memset( newData, 0, lineSize * abs( height));
    if ( var-> data)
       ic_stretch( var-> type, var-> data, var-> w, var-> h, 
                   newData, width, height, 
@@ -227,6 +261,7 @@ Image_reset_sv( Handle self, int new_type, SV * palette, Bool triplets)
 void
 Image_set( Handle self, HV * profile)
 {
+   dPROFILE;
    if ( pexist( conversion))
    {
       my-> set_conversion( self, pget_i( conversion));
@@ -355,6 +390,39 @@ Image_get_handle( Handle self)
    return newSVpv( buf, 0);
 }
 
+Color
+Image_get_nearest_color( Handle self, Color color)
+{
+   Byte pal;
+   RGBColor rgb, *pcolor;
+
+   if ( is_opt( optInDrawInfo) || is_opt( optInDraw))
+      return inherited get_nearest_color( self, color);
+
+   switch ( var-> type & imCategory) {
+   case imColor:
+      if (( var-> type & imBPP) > 8)
+         return color;
+      rgb. b = color         & 0xFF;
+      rgb. g = (color >> 8)  & 0xFF;
+      rgb. r = (color >> 16) & 0xFF;
+      break;
+   case imGrayScale:
+      rgb. r = rgb. g = rgb. b = (
+        (color & 0xFF) + 
+        ((color >> 8)  & 0xFF) + 
+        ((color >> 16) & 0xFF)
+      ) / 3;
+      break;
+   default:
+      return clInvalid; /* what else? */
+   }
+
+   pal    = cm_nearest_color( rgb, var-> palSize, var-> palette);
+   pcolor = var->palette + pal;
+   return ARGB( pcolor-> r, pcolor-> g, pcolor-> b);
+}
+
 SV *
 Image_data( Handle self, Bool set, SV * svdata)
 {
@@ -386,6 +454,7 @@ Image_data( Handle self, Bool set, SV * svdata)
 Bool
 Image_set_extended_data( Handle self, HV * profile)
 {
+   dPROFILE;
    void *data, *proc;
    STRLEN dataSize;
    int lineSize = 0, newType = -1, fixType, oldType = -1;
@@ -484,27 +553,110 @@ Image_set_extended_data( Handle self, HV * profile)
 GOOD_RETURN:   
    pdelete(data);
    return true;
-}   
+}
+
+static unsigned long 
+img_perlio_read( void * f, unsigned long bufsize, void * buffer)
+{
+#ifdef PerlIO
+    return PerlIO_read(( FileStream) f, buffer, bufsize);
+#else
+    return fread( buffer, bufsize, 1, ( FileStream) f);
+#endif
+}
+
+static unsigned long 
+img_perlio_write( void * f, unsigned long bufsize, void * buffer)
+{
+#ifdef PerlIO
+    return PerlIO_write( ( FileStream) f, buffer, bufsize);
+#else
+    return fwrite( buffer, bufsize, 1, ( FileStream) f);
+#endif
+}
+
+static unsigned long 
+img_perlio_seek( void * f, unsigned long offset, int whence)
+{
+#ifdef PerlIO
+    return PerlIO_seek( ( FileStream) f, offset, whence);
+#else
+    return fseek( ( FileStream) f, offset, whence);
+#endif
+}
+
+static unsigned long 
+img_perlio_tell( void * f)
+{
+#ifdef PerlIO
+    return PerlIO_tell( ( FileStream) f);
+#else
+    return ftell( ( FileStream) f);
+#endif
+}
+
+static int
+img_perlio_flush( void * f)
+{
+#ifdef PerlIO
+    return PerlIO_flush( ( FileStream) f);
+#else
+    return fflush( ( FileStream) f);
+#endif
+}
+
+static int
+img_perlio_error( void * f)
+{
+#ifdef PerlIO
+    return PerlIO_error( ( FileStream) f);
+#else
+    return ferror( ( FileStream) f);
+#endif
+}
 
 XS( Image_load_FROMPERL) 
 {
    dXSARGS;
    Handle self;
+   SV * sv;
    HV *profile;
    char *fn;
    PList ret;
    Bool err = false;
+   FileStream f = NULL;
+   ImgIORequest ioreq, *pioreq;
    char error[256];
 
    if (( items < 2) || (( items % 2) != 0))
       croak("Invalid usage of Prima::Image::load");
    
    self = gimme_the_mate( ST( 0));
-   fn   = ( char *) SvPV_nolen( ST( 1));
+
+   sv   = ST(1);
+   if ( SvROK(sv) && SvTYPE( SvRV( sv)) == SVt_PVGV)
+      f = IoIFP(sv_2io(ST(1)));
+
+   if ( f != NULL) {
+       pioreq        = &ioreq;
+       ioreq. handle = f;
+       ioreq. read   = img_perlio_read;
+       ioreq. write  = img_perlio_write;
+       ioreq. seek   = img_perlio_seek;
+       ioreq. tell   = img_perlio_tell;
+       ioreq. flush  = img_perlio_flush;
+       ioreq. error  = img_perlio_error;
+       fn            = NULL;
+   } else {
+       fn            = ( char *) SvPV_nolen( ST( 1));
+       pioreq        = NULL;
+   }
+   
    profile = parse_hv( ax, sp, items, mark, 2, "Image::load");
    if ( !pexist( className)) 
       pset_c( className, self ? my-> className : ( char*) SvPV_nolen( ST( 0)));
-   ret = apc_img_load( self, fn, profile, error);
+   pset_i( eventMask, self ? var-> eventMask2 : 0);
+   ret = apc_img_load( self, fn, pioreq, profile, error);
    sv_free(( SV *) profile);
    SPAGAIN;
    SP -= items;
@@ -538,6 +690,15 @@ XS( Image_load_FROMPERL)
    return;
 }   
 
+int
+Image_lineSize( Handle self, Bool set, int dummy)
+{
+   if ( set)
+      croak("Image::lineSize: attempt to write read-only property");
+
+   return var-> lineSize;
+}
+
 PList
 Image_load_REDEFINED( SV * who, char *filename, HV * profile)
 {
@@ -552,7 +713,7 @@ Image_load( SV * who, char *filename, HV * profile)
    char error[ 256];
    if ( !pexist( className)) 
       pset_c( className, self ? my-> className : ( char*) SvPV_nolen( who));
-   ret = apc_img_load( self, filename, profile, error);
+   ret = apc_img_load( self, filename, NULL, profile, error);
    return ret;
 }
 
@@ -565,14 +726,36 @@ XS( Image_save_FROMPERL)
    char *fn;
    int ret;
    char error[256];
+   FileStream f = NULL;
+   SV * sv;
+   ImgIORequest ioreq, *pioreq;
 
    if (( items < 2) || (( items % 2) != 0))
       croak("Invalid usage of Prima::Image::save");
    
    self = gimme_the_mate( ST( 0));
-   fn   = ( char *) SvPV_nolen( ST( 1));
+
+   sv   = ST(1);
+   if ( SvROK(sv) && SvTYPE( SvRV( sv)) == SVt_PVGV)
+      f = IoIFP(sv_2io(ST(1)));
+
+   if ( f != NULL) {
+       pioreq        = &ioreq;
+       ioreq. handle = f;
+       ioreq. read   = img_perlio_read;
+       ioreq. write  = img_perlio_write;
+       ioreq. seek   = img_perlio_seek;
+       ioreq. tell   = img_perlio_tell;
+       ioreq. flush  = img_perlio_flush;
+       ioreq. error  = img_perlio_error;
+       fn            = NULL;
+   } else {
+       fn            = ( char *) SvPV_nolen( ST( 1));
+       pioreq        = NULL;
+   }
+
    profile = parse_hv( ax, sp, items, mark, 2, "Image::save");
-   ret = apc_img_save( self, fn, profile, error);
+   ret = apc_img_save( self, fn, pioreq, profile, error);
    sv_free(( SV *) profile);
    SPAGAIN;
    SP -= items;
@@ -601,7 +784,7 @@ Image_save( SV * who, char *filename, HV * profile)
    char error[ 256];
    if ( !pexist( className)) 
       pset_c( className, self ? my-> className : ( char*) SvPV_nolen( who));
-   return apc_img_save( self, filename, profile, error);
+   return apc_img_save( self, filename, NULL, profile, error);
 }
 
 int
@@ -1188,6 +1371,7 @@ Image_map( Handle self, Color color)
          rop[i] = ropCopyPut;
          break;
       case ropNoOper:
+         r[i]. r = r[i]. g = r[i]. b = 0;
          break;   
       default: {   
          Color c = i ? my-> get_backColor( self) : my-> get_color( self);
@@ -1197,8 +1381,7 @@ Image_map( Handle self, Color color)
       }} 
                
       if (( type & imBPP) <= 8) {
-         b[0] = cm_nearest_color( r[0], var-> palSize, var-> palette);
-         b[1] = cm_nearest_color( r[1], var-> palSize, var-> palette);
+         b[i] = cm_nearest_color( r[i], var-> palSize, var-> palette);
       }
       
       switch ( rop[i]) {
@@ -1220,7 +1403,6 @@ Image_map( Handle self, Color color)
       }
    }         
 
-
    c. r = ( color >> 16) & 0xff;
    c. g = ( color >> 8) & 0xff;
    c. b = color & 0xff;
@@ -1324,7 +1506,9 @@ Image_codecs( SV * dummy)
    apc_img_codecs( p);  
    for ( i = 0; i < p-> count; i++) {
       PImgCodec c = ( PImgCodec ) p-> items[ i];
-      av_push( av, newRV_noinc(( SV *) apc_img_info2hash( c))); 
+      HV * profile = apc_img_info2hash( c);
+      pset_i( codecID, i);
+      av_push( av, newRV_noinc(( SV *) profile)); 
    }  
    plist_destroy( p);
    return newRV_noinc(( SV *) av); 
@@ -1344,6 +1528,40 @@ Image_put_image_indirect( Handle self, Handle image, int x, int y, int xFrom, in
    return ret;
 }
 
+long
+Image_add_notification( Handle self, char * name, SV * subroutine, Handle referer, int index)
+{
+   long id = inherited add_notification( self, name, subroutine, referer, index);
+   if ( id != 0) Image_reset_notifications( self);
+   return id;
+}
+
+void
+Image_remove_notification( Handle self, long id)
+{
+   inherited remove_notification( self, id);
+   Image_reset_notifications( self);
+}
+
+static void
+Image_reset_notifications( Handle self)
+{
+   int i;
+   PList  list;
+   void * ret[ 2];
+   int    cmd[ 2] = { IMG_EVENTS_HEADER_READY, IMG_EVENTS_DATA_READY };
+   var-> eventMask2 = var-> eventMask1;
+   if ( var-> eventIDs == nil) return;
+
+   ret[0] = hash_fetch( var-> eventIDs, "HeaderReady", 11);
+   ret[1] = hash_fetch( var-> eventIDs, "DataReady",   9);
+
+   for ( i = 0; i < 2; i++) {
+      if ( ret[i] == nil) continue;
+      list = var-> events + PTR2IV( ret[i]) - 1;
+      if ( list-> count > 0) var-> eventMask2 |= cmd[ i];
+   }
+}
 
 #ifdef __cplusplus
 }
